from rocolib.api.components.Component import Component
from rocolib.api.Function import Function


self = Component()

self.addSubcomponent("beam", "RectBeam")
self.addSubcomponent("header", "Header")
self.addSubcomponent("servoPins", "Cutout")

self.addParameter("brain", "nodeMCU", paramType="dimension")
self.addParameter("length", 90, paramType="length")
self.addParameter("width", 10, paramType="length")
self.addParameter("depth", 10, paramType="length")

### Set specific relationships between parameters
def getBrainParameter(p):
  return "brain", "getDim(x, '%s')" % p

self.addConstraint(("beam", "width"), "depth")
self.addConstraint(("beam", "depth"), "width")
self.addConstraint(("beam", "length"), "length")
self.addConstConstraint(("beam", "angle"), 90)

self.addConstraint(("beam", "minwidth"), *getBrainParameter("height"))
self.addConstraint(("beam", "mindepth"), *getBrainParameter("width"))
self.addConstraint(("beam", "minlength"), *getBrainParameter("length"))

self.addConstraint(("header", "nrows"), *getBrainParameter("nrows"))
self.addConstraint(("header", "ncols"), *getBrainParameter("ncols"))
self.addConstraint(("header", "rowsep"), *getBrainParameter("rowsep"))
self.addConstraint(("header", "colsep"), *getBrainParameter("colsep"))

self.addConstConstraint(("servoPins", "dy"), 8)
self.addConstConstraint(("servoPins", "dx"), 27)

self.addConnection(("beam", "face1"),
                   ("header", "decoration"),
                   mode="hole", offset=Function(params=("length", "brain"), fnstring="(7.5, -4.5 + 0.5*(x[0]-getDim(x[1], 'length')))"))

self.addConnection(("beam", "face1"),
                   ("servoPins", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("length", "brain"), fnstring="(-17.25, (0.5 * (x[0]-getDim(x[1], 'length')))-14)"))

self.inheritAllInterfaces("beam", prefix=None)

self.toLibrary("ESPBrains")
