from rocolib.api.components.Component import Component
from rocolib.api.Function import Function

#ESP3 STACK WITH PWM SERVO FEATHERWING
c = Component()

c.addSubcomponent("holder", "SimpleRectBeam")

#Dimensions of the ESP32 Stack
c.addParameter("length", 61, paramType="length")

c.addParameter("brains", "esp32stack", paramType="dimension")
c.addConstraint(("holder", "length"), "length")
c.addConstraint(("holder", "depth"), "brains", "60")
c.addConstraint(("holder", "width"), "brains", "getDim(x, 'width')")

#This parameter will change when the dimensions of the car or vehicle is changed
#so that it would not affect the ESP32 stack holder
c.addSubcomponent("header", "Header")

#OLED at the top
c.addConstraint(("header", "nrows"), "brains", "getDim(x, 'nrows')")
c.addConstraint(("header", "ncols"), "brains", "getDim(x, 'ncols')")
c.addConstraint(("header", "rowsep"), "brains", "getDim(x, 'rowsep')")
c.addConstraint(("header", "colsep"), "brains", "getDim(x, 'colsep')")

#Holes for servo and powersupply
c.addParameter("dy1", 18, parameterType="length")
for i in range(2):
    c.addSubcomponent("servohole%d" %i, "Cutout")
    c.addConstraint(("servohole%d" %i, "dy"), "dy1", "x+12")
    c.addConstraint(("servohole%d" %i, "dx"), "dy1", "x-5")

for i in range(4):
    c.addSubcomponent("pinhole%d" % i, "Cutout")

c.addConstraint(("pinhole0", "dy"), "dy1", "41")
c.addConstraint(("pinhole0", "dx"), "dy1", "1.2")
c.addConstraint(("pinhole1", "dy"), "dy1", "30")
c.addConstraint(("pinhole1", "dx"), "dy1", "1.2")

c.addConstraint(("pinhole2", "dy"), "dy1", "41")
c.addConstraint(("pinhole2", "dx"), "dy1", "1.2")
c.addConstraint(("pinhole3", "dy"), "dy1", "30")
c.addConstraint(("pinhole3", "dx"), "dy1", "1.2")

c.addConnection(("holder", "face0"),
                   ("header", "decoration"),
                   mode="hole", offset=Function(params=("length", "brains"), fnstring="(0, 0)"))

c.addConnection(("holder", "face2"),
                   ("header", "decoration"),
                   mode="hole", offset=Function(params=("length", "brains"), fnstring="(0, 0)"))

c.addConnection(("holder", "face3"),
                   ("servohole1", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("length"), fnstring="(0, 14)"))

c.addSubcomponent("powerhole", "Cutout")
c.addConstraint(("powerhole", "dy"), "dy1", "x+17")
c.addConstraint(("powerhole", "dx"), "dy1", "x")

c.addConnection(("holder", "face1"),
                   ("powerhole", "decoration"),
                   mode="hole", rotate=True, offset=Function(params=("length"), fnstring="(-5, 12)"))

c.inheritAllInterfaces("holder", prefix=None)

c.toLibrary("SubESP32Stack")
